function [modisData, modisDataStructure ] = modisClient( product, band, latitude, longitude, startDate, endDate, kmAboveBelow, kmLeftRight )
%
% modisClient: get modis data from the ORNL web service
%
% 	modisClient( ), with no arguments, returns a list of available products 
%
%
%	modisClient( product ) returns a list of all available bands for
%	the specified product. The argument 'product' is a string and should
%	be selected from the list returned by the preceding example.
%
%
%	modisClient( product, band, latitude, longitude ) returns a list of 
%       all available dates for the specified product and band at the specified
%       geographic coordinates. The argument 'band' is a string and should
%	be selected from the list returned by the preceding example. The
%	arguments latitude and longitude are floating point numbers.
%
%	
%	modisClient( product, band, latitude, longitude, start, end )
%	returns a structure containing MODIS data, for the specified 
%	product, band and geographic coordinates between the requested
%	start and end dates. The format of start and end is: YYYYJJJ
%	where is YYYY is the 4-digit year and JJJ is the 3-digit day of 
%	year. These dates do not need to correspond exactly to the dates
%	returned by the previous example (but only data for those dates
%	will be returned). 
%
%
%	modisClient( product, band, latitude, longitude, start, end, kmAboveBelow, kmLeftRight )
%	returns the same data structure as above but for a spatial extent 
%	defined by kmAboveBelow and kmLeftRight. These arguments are
%	floating point and have units of kilometers. The maximum extent
%	in either direction is 100km. The more data requested the slower
%	the server will respond!
%
%	See also modisMaskQC
%
%	For more information on the webservice:
%	http://daac.ornl.gov/MODIS/MODIS-menu/modis_webservice.html
%
%	For more information on the MODIS land products:
%	https://lpdaac.usgs.gov/lpdaac/products/modis_products_table/
%
% Thanks go to Dr. Tim Hill (St Andrews UK) for his addition of latitude 
% and longitude data planes.
%
% Tristan Quaife, Meteorology, University of Reading. (16/07/2012)
% t.l.quaife@reading.ac.uk


% VERSION 2.0 ===========================================================
%
% This is version 2.0 of the Matlab version of the ORNL DAAC Modis Client
% Major changes over the first version are:
%
% 1) A number of modified date arrays are attached to the structure to
%    make the users life easier when plotting
% 2) Various other bits if information append to the structure (such as
%    whether or not QC information has been downloaded
% 3) A resturcuring of the data order to make data access more intuitve
% 4) Addition of 2D latitude and longitude fields (with thanks to Tim Hill)

% modisClient: ==========================================================

    addpath([pwd '/utils/']);
    
    verbose = true ;
	webService = 'https://modis.ornl.gov/cgi-bin/MODIS/soapservice/MODIS_soapservice.wsdl' ;
	chunkSize = 8 ;
	
	modisData.data(1,1)=0 ;
	modisData.scale=1 ;
	modisData.dateList(1,1)=cell( 1 ) ;

	% generate the service handler object if it doesn't already exist
	if exist( 'myServiceHandlerService' ) ~= 2
		if verbose 
			disp( 'modisClient: modis service not found - generating...' );
		end
		createClassFromWsdl( webService );
	end
	
	% if it still doesn't exist, barf
	if exist( 'myServiceHandlerService' ) ~= 2
		if verbose
			disp( 'modisClient: unable to find modis service - exiting!' );
		end
		modisData = 0 ;
		return ;
	end

	%make new instance of the class
	modisServiceObj = myServiceHandlerService ;
	
	
	%if no args return the list of available products
	modisProds = getproducts( modisServiceObj ) ;
	if nargin==0	
		modisData=modisProds ;
		return;
	end


	%if one arg return the bands for that product
	
	if max( strcmp( product, modisProds ) ) == 0
		msg=strcat( 'modisClient: cannot find requested product "',product,'" on server. ' ) ;
		msg=strcat( msg, ' Available products are: ' ) ;
		for i = 1:length( modisProds ),
			msg = [ msg, ' ' char( modisProds( i ) ) ] ;
		end
		error( msg )
	end
			
	modisProdBands = getbands( modisServiceObj, product ) ;	
	
	if nargin==1	
		modisData=modisProdBands ;
		return;
	end


	%never two args - must specify lat AND long
	if nargin==3||nargin==2	
		error( 'modisClient: requires specification of band, latitude and longitude' ) ;
	end


	%check specified band exists
	if max( strcmp( band, modisProdBands ) ) == 0
		msg=strcat( 'modisClient: cannot find requested band "',band,'" on server. ' ) ;
		msg=strcat( msg, ' Available bands are: ' ) ;
		for i = 1:length( modisProdBands ),
			msg = [ msg, ' ' char( modisProdBands( i ) ) ] ;
		end
		error( msg )
	end


	%if three args return list of valid date strings
	modisProdDates = getdates( modisServiceObj, latitude, longitude, product ) ;
	if nargin==4	
		modisData=modisProdDates ;
		return;
	end


	%never five args - must always specify start and end dates 
	if nargin==5	
		error( 'modisClient: requires specification start AND end date' ) ;
	end



	%spatial dimensions default to 0
	if nargin==6
		kmAboveBelow=0 ; 
		kmLeftRight=0 ;
	end
	if nargin==7
		kmLeftRight=0 ;
	end

	if kmAboveBelow>100||kmAboveBelow<0||kmLeftRight>100||kmLeftRight<0
		error( 'modisClient: spatial dimensions of subset must be in the range 0-100' )
	end

	
	%if we have got this far it means that the user has called the functions with
	%all the information required to pull some data from the server

	n=1;
	[startDateFmtd, endDateFmtd ] = getModisDateChunk( modisProdDates, chunkSize, n, startDate, endDate ) ;

	while strcmp( char( startDateFmtd ), '0') ~= true && strcmp( char( endDateFmtd ), '0' ) ~= true

		if verbose
			disp( [ num2str(n) ' ' startDateFmtd ' ' endDateFmtd ] ) ;
		end

		modisDataStructure = getsubset( modisServiceObj, latitude, longitude, product, band, startDateFmtd, endDateFmtd, kmAboveBelow, kmLeftRight );

		if n==1 
			modisData.xllcorner=modisDataStructure.xllcorner ;
			modisData.yllcorner=modisDataStructure.yllcorner ;
			modisData.cellsize=modisDataStructure.cellsize ;
			modisData.nrows=modisDataStructure.nrows ;
			modisData.ncols=modisDataStructure.ncols ;
			modisData.band=modisDataStructure.band ;
			modisData.units=modisDataStructure.units ;
			modisData.scale=modisDataStructure.scale ;
			modisData.longitude=modisDataStructure.longitude ;
			modisData.latitude=modisDataStructure.latitude ;
            
            %additional stuff in V2
            modisData.product=product;
            modisData.userStartDate=startDate;
            modisData.userEndDate=endDate;
            modisData.userkmAboveBelow=kmAboveBelow;
            modisData.userkmLeftRight=kmLeftRight;
            modisData.gotQC=false;
		end


		%n.b. there is a bit of a work around in play here
		%when startDateFmtd == endDateFmtd we have to handle
		%the string slightly differently. There's probably a more elegant
		%way to solve this

		X=length( modisDataStructure.subset ) ; 
		if startDateFmtd == endDateFmtd
			X = 1 ;
		end

		for i = 1:X
			%disp( char( modisDataStructure.subset ) );
			if startDateFmtd == endDateFmtd
				subsetStringArray = strsplit( ',', strtrim( char( modisDataStructure.subset ) ) ) ;
			else
				subsetStringArray = strsplit( ',', strtrim( char( modisDataStructure.subset( i ) ) ) ) ;
            end
            
            %process date information to provide the user a variety of
            %different formats to access the information in
            modisData.dateList( i+(n-1)*chunkSize )=subsetStringArray( 3 );
            dateInt=str2num( char( regexprep( subsetStringArray( 3 ), 'A', '' ) ) );
            dateYear=int32(dateInt/1000.);
            dateDoY=int32(1000.*(dateInt/1000.-double(int32(dateInt/1000.))));
            dateDec=double(dateYear)+double(dateDoY)/365.;
			
            modisData.dateYYYYJJJ( i+(n-1)*chunkSize )=dateInt;   
			modisData.dateDecimal( i+(n-1)*chunkSize )=dateDec;
            modisData.dateDoY( i+(n-1)*chunkSize )=dateDoY;
            modisData.dateYear( i+(n-1)*chunkSize )=dateYear;
            
            
			for j = 6:length( subsetStringArray )
				modisData.data( i+(n-1)*chunkSize,j-5 ) = str2num( char( subsetStringArray( j ) ) ) ;
			end
		end

		n=n+1;
		[startDateFmtd, endDateFmtd] = getModisDateChunk( modisProdDates, chunkSize, n, startDate, endDate ) ;
	

    end

    % Tim's code:
    
    %Find the number of records in time
    recN = size(modisData.data,1); 
    %Flip dimensions for reshape
    modisData.data = modisData.data'; 
    %Reshape the data to be a nrows by ncols map.
    modisData.data = reshape(modisData.data,modisData.nrows,modisData.ncols,recN); 
    %Transpose the map
    modisData.data = permute(modisData.data,[2,1,3]); 

    %Note that: 
    %[yx]llcorner is the corner of lower left pixel in Sinusoidal projection
    %Build the xll and yll grid assuming "cellsize" is constant over the map:
    [r,c]=meshgrid(1:modisData.nrows,1:modisData.ncols);
    %Increments from the lower-left of lowest and leftest pixel, to the middle of the pixels
    r=(r-0.5)*modisData.cellsize; 
    c=(flipud(c)-0.5)*modisData.cellsize;
    map.xll = double(repmat(modisData.xllcorner,modisData.nrows,modisData.ncols)+r);
    map.yll = double(repmat(modisData.yllcorner,modisData.nrows,modisData.ncols)+c);

    %Convert to lat longs
    [modisData.latMap,modisData.lonMap]=yllxll2latlon(map.yll,map.xll);
    
end

